| package |
package := Package name: 'XmlRpc Client'.
package paxVersion: 0;
	basicComment: '"The XMl-RPC spec indicates which data types can be sent to/from the server. Parameters in remote procedure calls and return types are restricted to these types. These data types and the Smalltalk types they mapped to are:

XML-RPC type	-> Smalltalk type
<int>					SmallInteger
<boolean>				true, false
<string>				String
<double>				Float
<dateTime.ios8601>	Timestamp
<base64>				ByteArray
<struct>				Dictionary (key = member name, value = member value)
<array>				OrderedCollection

The mappings from Smalltalk to XML-RPC are the same except for:

Smalltalk type	-> XML-RPC type
Integer (including Large)	<int>
Number					<double>
SequenceableCollection	<array>
Object						<object>	(extended type not in XML-RPC spec)
nil							<nil>		(extended type not in XML-RPC spec)

The XML-RPC spec indicates the range of <int> (four bytes, signed) and <double> (double precision). If a Smalltalk XML-RPC client/server talks to a non-Smalltalk server/client the extended range of <int> and <double> could cause problems. If this is a issue the Smalltalk client/Sever needs to restrict the values it sends.

This implementation extends the XML-RPC spec by adding an Object tag. Handles only objects with instance variables and no circular references. The Object tag is experiemental. It will not work with clients/servers in other languages. If you wish to turn it off, replace the code in Object>>printRcpXmlOn: with self subclassResponsibility. 
'.

package basicPackageVersion: '1.0'.


package classNames
	add: #MockXmlRpcClient;
	add: #XmlRpcClient;
	add: #XmlRpcFault;
	add: #XmlRpcMethodCall;
	add: #XmlRpcMethodResult;
	add: #XmlRpcParseError;
	add: #XmlRpcParser;
	add: #XmlRpcProxy;
	add: #XmlRpcRemoteException;
	yourself.

package methodNames
	add: #Behavior -> #xmlRpcNew:;
	add: #ByteArray -> #asBase64MimeEncoding;
	add: #ByteArray -> #printXmlRpcOn:;
	add: #Date -> #printXmlRpcOn:;
	add: #Dictionary -> #printXmlRpcOn:;
	add: #False -> #printXmlRpcOn:;
	add: #Float -> #printXmlRpcOn:;
	add: #Integer -> #printXmlRpcOn:;
	add: #Number -> #printXmlRpcOn:;
	add: #Object -> #asXmlRpc;
	add: #Object -> #isXmlRpcFault;
	add: #Object -> #printXmlRpcOn:;
	add: #SequenceableCollection -> #printXmlRpcOn:;
	add: #String -> #addHttpProtocol;
	add: #String -> #asHttpPath;
	add: #String -> #asInteger;
	add: #String -> #decodeBase64;
	add: #String -> #decodeXmlRpc;
	add: #String -> #dropSeparators;
	add: #String -> #isXmlRpcArray;
	add: #String -> #isXmlRpcBase64;
	add: #String -> #isXmlRpcBoolean;
	add: #String -> #isXmlRpcDateTime;
	add: #String -> #isXmlRpcDouble;
	add: #String -> #isXmlRpcInt;
	add: #String -> #isXmlRpcNil;
	add: #String -> #isXmlRpcObject;
	add: #String -> #isXmlRpcString;
	add: #String -> #isXmlRpcStruct;
	add: #String -> #printXmlRpcOn:;
	add: #String -> #trimSeparators;
	add: #Time -> #printXmlRpcOn:;
	add: #TimeStamp -> #printXmlRpcOn:;
	add: #True -> #printXmlRpcOn:;
	add: #UndefinedObject -> #printXmlRpcOn:;
	add: #WriteStream -> #nextPutXmlRpc:;
	add: 'ByteArray class' -> #fromBase64MimeEncoding:;
	add: 'TimeStamp class' -> #fromISO8601:;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	yourself).

package!

"Class Definitions"!

Object subclass: #MockXmlRpcClient
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #XmlRpcClient
	instanceVariableNames: 'contents url'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #XmlRpcFault
	instanceVariableNames: 'errorCode description'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #XmlRpcMethodCall
	instanceVariableNames: 'methodName arguments'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #XmlRpcMethodResult
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #XmlRpcParser
	instanceVariableNames: 'source'
	classVariableNames: 'PredefinedEntities'
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #XmlRpcProxy
	instanceVariableNames: 'namespace client'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Error subclass: #XmlRpcParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Error subclass: #XmlRpcRemoteException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!Behavior methodsFor!

xmlRpcNew: aDictionary

	"Answer a new instance of the receiver instance variables initialized with values in the 
	dictionary. Keys are the names of the variables. "

	| object names |

	object := self new.
	names := self instVarNames.
	names
		keysAndValuesDo:
			[ :key :value | object instVarAt: key put: (aDictionary at: value ifAbsent: [ nil ]) ].
	^object! !
!Behavior categoriesFor: #xmlRpcNew:!public! !

!ByteArray methodsFor!

asBase64MimeEncoding

	^''!

printXmlRpcOn: aStream 
	"RPC XML Strings can not contain $< & $&"

	aStream
		nextPutAll: '<base64>';
		nextPutAll: self asBase64MimeEncoding;
		nextPutAll: '</base64>'! !
!ByteArray categoriesFor: #asBase64MimeEncoding!printing!public! !
!ByteArray categoriesFor: #printXmlRpcOn:!printing!public! !

!ByteArray class methodsFor!

fromBase64MimeEncoding: aString! !
!ByteArray class categoriesFor: #fromBase64MimeEncoding:!public! !

!Date methodsFor!

printXmlRpcOn: aStream 
	self date printOn: aStream format: 'yyyyMMdd'! !
!Date categoriesFor: #printXmlRpcOn:!public! !

!Dictionary methodsFor!

printXmlRpcOn: aStream 
	aStream
		nextPutAll: '<struct>';
		cr.
	self keysAndValuesDo: 
			[:key :value | 
			aStream
				tab;
				nextPutAll: '<member>';
				nextPutAll: '<name>';
				nextPutAll: key;
				nextPutAll: '</name>';
				nextPutAll: '<value>';
				nextPutAll: value asXmlRpc;
				nextPutAll: '</value></member>';
				cr].
	aStream nextPutAll: '</struct>'! !
!Dictionary categoriesFor: #printXmlRpcOn:!public! !

!False methodsFor!

printXmlRpcOn: aStream 

	aStream nextPutAll: '<boolean>0</boolean>'! !
!False categoriesFor: #printXmlRpcOn:!public! !

!Float methodsFor!

printXmlRpcOn: aStream 
	"No exponents allowed in rpc xml representation of floats"

	| integerPart fractionalPart |
	integerPart := self integerPart asInteger abs printString.
	fractionalPart := self fractionPart abs printString.
	"remove leading 0 in 0.1233"
	aStream
		nextPutAll: '<double>';
		nextPutAll: (self < 0 ifTrue: ['-'] ifFalse: ['']);
		nextPutAll: integerPart;
		nextPutAll: (fractionalPart copyFrom: 2 to: fractionalPart size);
		nextPutAll: '</double>'! !
!Float categoriesFor: #printXmlRpcOn:!public! !

!Integer methodsFor!

printXmlRpcOn: aStream

	aStream
		nextPutAll: '<int>';
		print: self;
		nextPutAll: '</int>'! !
!Integer categoriesFor: #printXmlRpcOn:!public! !

!Number methodsFor!

printXmlRpcOn: aStream

	aStream nextPutXmlRpc: self asFloat ! !
!Number categoriesFor: #printXmlRpcOn:!public! !

!Object methodsFor!

asXmlRpc
	| aStream |
	aStream := (String new: 128) writeStream.
	self printXmlRpcOn: aStream.
	^aStream contents!

isXmlRpcFault
	^false!

printXmlRpcOn: aStream 
	"Turn objects to XML-RPC Xml. The Object tag is experimental. To turn if off 
	replace the code below with self subclassResponsibilty"

	| names varData |
	names := self class instVarNames.
	varData := Dictionary new.
	names keysAndValuesDo: [:index :name | varData at: name put: (self instVarAt: index)].
	aStream
		nextPutAll: '<object><class>';
		nextPutAll: self class name asString;
		nextPutAll: '</class><instanceVariables>';
		nextPutXmlRpc: varData;
		nextPutAll: '</instanceVariables></object>'! !
!Object categoriesFor: #asXmlRpc!public! !
!Object categoriesFor: #isXmlRpcFault!public! !
!Object categoriesFor: #printXmlRpcOn:!public! !

!SequenceableCollection methodsFor!

printXmlRpcOn: aStream 
	aStream
		nextPutAll: '<array><data>';
		cr.
	self do: [ :each | 
		aStream
			tab;
			nextPutAll: '<value>';
			nextPutXmlRpc: each;
			nextPutAll: '</value>';
			cr ].
	aStream nextPutAll: '</data></array>'! !
!SequenceableCollection categoriesFor: #printXmlRpcOn:!public! !

!String methodsFor!

addHttpProtocol

	^(self beginsWith: 'http://')
		ifTrue: [ self ]
		ifFalse: [ 'http://' , self ]!

asHttpPath

	| trimmed |
	trimmed := self trimBlanks.
	^trimmed first = $/
		ifTrue:[ trimmed ]
		ifFalse:[ '/' , trimmed ]!

asInteger 
	"Answer the number created by interpreting the receiver as the string
	representation of a number.  First remove any leading seperators."
	| stream |
	stream := self readStream.
	stream skipSeparators.
	^Integer readFrom: stream!

decodeBase64
	"If self is the base64 mime encoding of byte array X, then return X. See rfc 1521
	 http://www.ietf.org/rfc/rfc1521.txt. section 5.2 for the base 64 encoding algorithm"

	^ByteArray fromBase64String: self!

decodeXmlRpc
	"Three pass in worst case. If find is slow and/or have time reduce to single pass"

	| decoded encoded entity |
	(self includes: $;) ifFalse: [^self].
	decoded := WriteStream on: (String new: self size).
	encoded := ReadStream on: self.
	decoded nextPutAll: (encoded upTo: $&).
	[encoded atEnd] whileFalse: 
			[entity := encoded upTo: $;.
			decoded nextPutAll: (XmlRpcParser entityAt: entity ifAbsent: ['&' , entity , ';']).
			decoded nextPutAll: (encoded upTo: $&)].
	^decoded contents!

dropSeparators
	"Return a copy of the receiver with all whitespace removed"

	"     '   a b	c d   ' dropSeparators      "

	^self select: [:c | c isSeparator not]!

isXmlRpcArray

	^self = 'array' or: [self = 'array /']!

isXmlRpcBase64

	^self = 'base64' or: [self = 'base64 /']!

isXmlRpcBoolean

	^self = 'boolean' or: [self = 'boolean /']!

isXmlRpcDateTime

	^self = 'dateTime.iso8601' or: [self = 'dateTime.iso8601 /']!

isXmlRpcDouble

	^self = 'double' or: [self = 'double /']!

isXmlRpcInt

	^(self = 'int') | (self = 'int /') | (self = 'i4') | (self = 'i4 /')!

isXmlRpcNil

	^self = 'nil' or: [self = 'nil /']!

isXmlRpcObject

	^self = 'object' or: [ self = 'object /' ]!

isXmlRpcString

	^self = 'string' or: [ self = 'string /' ]!

isXmlRpcStruct

	^self = 'struct' or: [ self = 'struct /' ]!

printXmlRpcOn: aStream 
	"RPC XML Strings can not contain $< & $&"

	aStream nextPutAll: '<string>'.
	self do: 
			[:each | 
			each = $< 
				ifTrue: [aStream nextPutAll: '&lt;']
				ifFalse: 
					[each = $& 
						ifTrue: [aStream nextPutAll: '&amp;']
						ifFalse: [each = $> ifTrue: [aStream nextPutAll: '&gt;'] ifFalse: [aStream nextPut: each]]]].
	aStream nextPutAll: '</string>'!

trimSeparators
	"return a copy of the reciever without any spaces 
	on front or back. Protect against all blanks"

	| start stop |
	start := 1.
	stop := self size.
	[ 
		start > stop ifTrue: [ ^'' ].
		(self at: start) isSeparator] whileTrue: [ start := start + 1 ].
	[(self at: stop) isSeparator] whileTrue: [ stop := stop - 1 ].
	^self copyFrom: start to: stop! !
!String categoriesFor: #addHttpProtocol!public! !
!String categoriesFor: #asHttpPath!public! !
!String categoriesFor: #asInteger!public! !
!String categoriesFor: #decodeBase64!public! !
!String categoriesFor: #decodeXmlRpc!public! !
!String categoriesFor: #dropSeparators!public! !
!String categoriesFor: #isXmlRpcArray!public! !
!String categoriesFor: #isXmlRpcBase64!public! !
!String categoriesFor: #isXmlRpcBoolean!public! !
!String categoriesFor: #isXmlRpcDateTime!public! !
!String categoriesFor: #isXmlRpcDouble!public! !
!String categoriesFor: #isXmlRpcInt!public! !
!String categoriesFor: #isXmlRpcNil!public! !
!String categoriesFor: #isXmlRpcObject!public! !
!String categoriesFor: #isXmlRpcString!public! !
!String categoriesFor: #isXmlRpcStruct!public! !
!String categoriesFor: #printXmlRpcOn:!public! !
!String categoriesFor: #trimSeparators!public! !

!Time methodsFor!

printXmlRpcOn: aStream 
	self time printOn: aStream format: 'hh:mm:ss'! !
!Time categoriesFor: #printXmlRpcOn:!public! !

!TimeStamp methodsFor!

printXmlRpcOn: aStream 
	"'yyyymmddThh:mm:ss'"

	aStream
		nextPutAll: '<dateTime.iso8601>';
		nextPutXmlRpc: self date;
		nextPut: $T;
		nextPutXmlRpc: self time;
		nextPutAll: '</dateTime.iso8601>'! !
!TimeStamp categoriesFor: #printXmlRpcOn:!printing!private! !

!TimeStamp class methodsFor!

fromISO8601: aString 
	"TimeStamp fromISO8601: '1998-07-17T14:08:55' "
	"TimeStamp fromISO8601: '19980717T14:08:55' "

	| year month day date time stream |
	stream := aString readStream.
	(aString includes: $-) 
		ifTrue: [
			year := (stream upTo: $-) asInteger.
			month := (stream upTo: $-) asInteger.
			day := (stream next: 2) asInteger ]
		ifFalse: [
			year := (stream next: 4) asInteger.
			month := (stream next: 2) asInteger.
			day := (stream next: 2) asInteger ].
	stream skip: 1.
	time := Time readFrom: stream upToEnd readStream.
	date := Date 
		newDay: day
		monthNumber: month
		year: year.
	^self date: date time: time! !
!TimeStamp class categoriesFor: #fromISO8601:!public! !

!True methodsFor!

printXmlRpcOn: aStream 

	aStream nextPutAll: '<boolean>1</boolean>'! !
!True categoriesFor: #printXmlRpcOn:!public! !

!UndefinedObject methodsFor!

printXmlRpcOn: aStream

	aStream nextPutAll: '<nil />'! !
!UndefinedObject categoriesFor: #printXmlRpcOn:!public! !

!WriteStream methodsFor!

nextPutXmlRpc: anObject 
	anObject printXmlRpcOn: self! !
!WriteStream categoriesFor: #nextPutXmlRpc:!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

MockXmlRpcClient guid: (GUID fromString: '{35FE8C53-E1D0-4F83-9A22-5883FF3B17DE}')!
MockXmlRpcClient comment: ''!
!MockXmlRpcClient categoriesForClass!Kernel-Objects! !
!MockXmlRpcClient methodsFor!

contents: aString

	contents := aString!

execute

	contents inspect!

methodName: methodNameString arguments: anOrderedCollection

	"return an XML-RPC request for an XML-RPC server to execute the indicated
	 metthod (with arguments) of the object associated with given label."

	| requestedMethod |

	requestedMethod := XmlRpcMethodCall
		methodName: methodNameString
		arguments: anOrderedCollection.
	self contents: requestedMethod asXmlRpc!

perform: aRemoteMethodString withArguments: aCollection

	"return an XML-RPC request for an XML-RPC server to execute the indicated
	 metthod (with arguments) of the object associated with given label."

	| requestedMethod |

	requestedMethod := XmlRpcMethodCall
		methodName: aRemoteMethodString
		arguments: aCollection.
	self contents: requestedMethod asXmlRpc.
	^self execute! !
!MockXmlRpcClient categoriesFor: #contents:!private! !
!MockXmlRpcClient categoriesFor: #execute!public! !
!MockXmlRpcClient categoriesFor: #methodName:arguments:!public! !
!MockXmlRpcClient categoriesFor: #perform:withArguments:!public! !

XmlRpcClient guid: (GUID fromString: '{5D7FEF20-2C71-43E2-9A25-DBCDACDE04EA}')!
XmlRpcClient comment: 'Simple HttpRequest with extensions for XML-RPC.

Instance Variables:
	httpVersion	<String>	HTTP/1.0, HTTP/1.1
	method	<String>	POST, GET, etc
	requestPath	<String>	url of the requested page. XML-RPC uses /RPC2 for this, but unless a request is going through a Web server this is ignored.
'!
!XmlRpcClient categoriesForClass!Kernel-Objects! !
!XmlRpcClient methodsFor!

contents

	^contents!

contents: anObject

	contents := anObject!

execute

	^(XmlRpcMethodResult from: self executeRequest) result!

executeRequest

	| request |

	request := self requestClass new.
	request
		open: 'POST'
		bstrUrl: url
		varAsync: false
		bstrUser: nil
		bstrPassword: nil.
	request send: self contents.
	^request responseText readStream!

initialize

	"Set the required headers for XmlRpc"
"
	self contentType: 'text/xml'.
	self fieldValueAt: 'User-Agent' put: 'Dolphin RPC Client'.
"
!

methodName: methodNameString

	"return an XML-RPC request for an XML-RPC server to execute the indicated
	 metthod (no arguments) of the object associated with given label."

	^self methodName: methodNameString arguments: nil!

methodName: methodNameString argument: anObject

	"return an XML-RPC request for an XML-RPC server to execute the indicated
	 metthod (with one argument) of the object associated with given label."

	^self methodName: methodNameString arguments: (Array with: anObject)!

methodName: methodNameString arguments: anOrderedCollection

	"return an XML-RPC request for an XML-RPC server to execute the indicated
	 metthod (with arguments) of the object associated with given label."

	| requestedMethod |

	requestedMethod := XmlRpcMethodCall
		methodName: methodNameString
		arguments: anOrderedCollection.
	self contents: requestedMethod asXmlRpc!

perform: aRemoteMethodString

	^self
		methodName: aRemoteMethodString;
		execute!

perform: aRemoteMethodString with: argument

	self methodName: aRemoteMethodString argument: argument.
	^self execute!

perform: aRemoteMethodString with: argument1 with: argument2

	^self perform: aRemoteMethodString withArguments: (Array with: argument1 with: argument2)!

perform: aRemoteMethodString withArguments: aCollection

	self methodName: aRemoteMethodString arguments: aCollection.
	^self execute!

requestClass
	
	^Smalltalk at: #IXMLHttpRequest ifAbsent: [ Smalltalk at: #MSXMLIXMLHttpRequest ]!

url: serviceUrl 
	url := serviceUrl! !
!XmlRpcClient categoriesFor: #contents!accessing!private! !
!XmlRpcClient categoriesFor: #contents:!accessing!private! !
!XmlRpcClient categoriesFor: #execute!public! !
!XmlRpcClient categoriesFor: #executeRequest!public! !
!XmlRpcClient categoriesFor: #initialize!public! !
!XmlRpcClient categoriesFor: #methodName:!public! !
!XmlRpcClient categoriesFor: #methodName:argument:!public! !
!XmlRpcClient categoriesFor: #methodName:arguments:!public! !
!XmlRpcClient categoriesFor: #perform:!public! !
!XmlRpcClient categoriesFor: #perform:with:!public! !
!XmlRpcClient categoriesFor: #perform:with:with:!public! !
!XmlRpcClient categoriesFor: #perform:withArguments:!public! !
!XmlRpcClient categoriesFor: #requestClass!public! !
!XmlRpcClient categoriesFor: #url:!public! !

!XmlRpcClient class methodsFor!

addTwoExample
	"self addTwoExample"

	^self validatorProxy 
		perform: 'examples.addtwo' 
		with: 5
		with: 3!

getStateName: anInteger 
	"Uses a userland server to return a name of a state"

	"self getStateName: 8"

	| proxy |
	proxy := self url: 'http://betty.userland.com/RPC2'.
	^proxy perform: 'examples.getStateName' with: anInteger!

host: aString 
	^self 
		host: aString
		port: 80
		path: 'RPC2'!

host: aString path: aPathString 
	^self 
		host: aString
		port: 80
		path: aPathString!

host: aString port: anInteger 
	^self 
		host: aString
		port: anInteger
		path: 'RPC2'!

host: aString port: anInteger path: aPathString 
	^self url: aString , ':' , anInteger printString , aPathString asHttpPath!

moderateSizeArrayCheckExample: anOrderedCollectionOfStrings
	"See http://www.xmlrpc.com/validator1Docs for info on this remote method.
	Returns first element concatenated with last element"

	"self moderateSizeArrayCheckExample: #('a' 'dog' 'cat' )"

	^self validatorProxy 
		perform: 'validator1.moderateSizeArrayCheck' 
		with: anOrderedCollectionOfStrings.!

multipleArgumentsExample
	"See http://www.xmlrpc.com/validator1Docs for info on this remote method.
	Returns collection of 6 different types"

	"self multipleArgumentsExample"

	| arguments |
	arguments := OrderedCollection new.
	arguments
		add: 10;
		add: true;
		add: 'aString';
		add: 3.1415;
		add: TimeStamp now;
		add: #[ 1 2 3].
		
	^self validatorProxy 
		perform: 'validator1.manyTypesTest' 
		withArguments: arguments.!

simpleStructReturnExample
	"See http://www.xmlrpc.com/validator1Docs for info on this remote method.
	Returns 10, 100, 1000 time the value sent"
	"self simpleStructReturnExample"

	^self validatorProxy 
		perform: 'validator1.simpleStructReturnTest' 
		with: 5.!

spellCheck
	"See http://www.stuffeddog.com/speller/doc/rpc.html for information on the server used."
	"self spellCheck"

	| client |
	client := self url: 'http://www.stuffeddog.com/speller/speller-rpc.cgi'.
	^client 
		perform: 'speller.spellCheck'
		with: 'This is a ccat'
		with: Array new!

time
	"See http://www.xmlrpc.com/currentTime for information on the server"
	"self time"

	^(self host: 'time.xmlrpc.com') 
		perform: 'currentTime.getCurrentTime'!

url: aUrlString 
	^(self new)
		url: aUrlString addHttpProtocol;
		yourself!

validatorProxy
	"Return proxy to the XML-RPC validator server. See http://www.dscpl.com.au/xmlrpc-debugger.php
	This proxy is for a well know XML-RPC server which runs all the time"

	^self url: 'http://xmlrpc.usefulinc.com/demo/server.php'! !
!XmlRpcClient class categoriesFor: #addTwoExample!examples!public! !
!XmlRpcClient class categoriesFor: #getStateName:!examples!public! !
!XmlRpcClient class categoriesFor: #host:!public! !
!XmlRpcClient class categoriesFor: #host:path:!public! !
!XmlRpcClient class categoriesFor: #host:port:!public! !
!XmlRpcClient class categoriesFor: #host:port:path:!public! !
!XmlRpcClient class categoriesFor: #moderateSizeArrayCheckExample:!examples!public! !
!XmlRpcClient class categoriesFor: #multipleArgumentsExample!examples!public! !
!XmlRpcClient class categoriesFor: #simpleStructReturnExample!examples!public! !
!XmlRpcClient class categoriesFor: #spellCheck!examples!public! !
!XmlRpcClient class categoriesFor: #time!examples!public! !
!XmlRpcClient class categoriesFor: #url:!public! !
!XmlRpcClient class categoriesFor: #validatorProxy!examples!public! !

XmlRpcFault guid: (GUID fromString: '{F43E0BE2-4D32-44BE-9949-406733154847}')!
XmlRpcFault comment: 'XML-RPC uses XML in the body of HTTP message to send back errors (faults) from RPCs. 

Instance Variables:
	description	<String>	description of the fault
	errorCode	<Integer>	number of the error. XML-RPC spec does not specify values. See below

-1 = low level error
0 = no error
1 = server does not contain indicated object
2 = requested method does not exist
3 = not correct number of arguments for method
'!
!XmlRpcFault categoriesForClass!Kernel-Objects! !
!XmlRpcFault methodsFor!

description

	^description!

description: aString

	description := aString!

errorCode

	^errorCode!

errorCode: anInteger

	errorCode := anInteger!

isXmlRpcFault

	^true!

printXmlRpcOn: aStream

	| data |

	data := (Dictionary new)
		at: 'faultCode' put: errorCode;
		at: 'faultString' put: description;
		yourself.
	aStream
		nextPutAll: '<fault><value>';
		nextPutXmlRpc: data;
		nextPutAll: '</value></fault>'! !
!XmlRpcFault categoriesFor: #description!public! !
!XmlRpcFault categoriesFor: #description:!public! !
!XmlRpcFault categoriesFor: #errorCode!public! !
!XmlRpcFault categoriesFor: #errorCode:!public! !
!XmlRpcFault categoriesFor: #isXmlRpcFault!public! !
!XmlRpcFault categoriesFor: #printXmlRpcOn:!public! !

!XmlRpcFault class methodsFor!

errorCode: anInteger description: aString
	^super new
		errorCode: anInteger;
		description: aString! !
!XmlRpcFault class categoriesFor: #errorCode:description:!public! !

XmlRpcMethodCall guid: (GUID fromString: '{6AEC900E-65D4-4F48-A749-076DF848FFFA}')!
XmlRpcMethodCall comment: 'XmlRpcMethodCall contains needed information about a remote method call request. 
Translates the information from/to XML specified by the XML-RPC spec

Instance Variables:
	arguments	<Collection of: Object>	arguments for remote method call, nil or empty collection for no arguments.
	methodName	<String>	name of the remote method to call
	objectLabel	<String>	key of the server object which is to execute this remote method

'!
!XmlRpcMethodCall categoriesForClass!Kernel-Objects! !
!XmlRpcMethodCall methodsFor!

arguments

	^arguments!

arguments: aCollection

	arguments := aCollection!

asXmlRpc

	| aStream numberOfArguments |

	numberOfArguments := arguments isNil ifTrue: [ 1 ] ifFalse: [ arguments size ].
	aStream := WriteStream on: (String new: numberOfArguments * 100 + 100).
	self printXmlRpcOn: aStream.
	^aStream contents!

from: aStream

	| parser |

	parser := XmlRpcParser on: aStream.
	parser processMethodCallFor: self!

method

	^methodName asSymbol!

methodName

	^methodName!

methodName: aString
	methodName := aString!

methodNameXmlOn: aStream

	aStream
		nextPutAll: '<methodName>';
		nextPutAll: methodName asString;
		nextPutAll: '</methodName>'!

parameterXmlOn: aStream

	(arguments isNil or: [ arguments isEmpty ]) ifTrue: [ ^nil ].
	aStream
		nextPutAll: '<params>';
		cr;
		tab.
	arguments
		do:
			[ :each | 
			aStream
				nextPutAll: '<param><value>';
				nextPutXmlRpc: each;
				nextPutAll: '</value></param>' ].
	aStream
		cr;
		nextPutAll: '</params>'!

printXmlRpcOn: aStream

	aStream
		nextPutAll: XmlRpcParser xmlVersion;
		nextPutAll: '<methodCall>'.
	self
		methodNameXmlOn: aStream;
		parameterXmlOn: aStream.
	aStream nextPutAll: '</methodCall>'!

serverLabel

	^(methodName includes: $.)
		ifTrue: [ (methodName tokensBasedOn: $.) first ]
		ifFalse: [ '' ]! !
!XmlRpcMethodCall categoriesFor: #arguments!public! !
!XmlRpcMethodCall categoriesFor: #arguments:!public! !
!XmlRpcMethodCall categoriesFor: #asXmlRpc!public! !
!XmlRpcMethodCall categoriesFor: #from:!public! !
!XmlRpcMethodCall categoriesFor: #method!public! !
!XmlRpcMethodCall categoriesFor: #methodName!public! !
!XmlRpcMethodCall categoriesFor: #methodName:!public! !
!XmlRpcMethodCall categoriesFor: #methodNameXmlOn:!public! !
!XmlRpcMethodCall categoriesFor: #parameterXmlOn:!public! !
!XmlRpcMethodCall categoriesFor: #printXmlRpcOn:!public! !
!XmlRpcMethodCall categoriesFor: #serverLabel!public! !

!XmlRpcMethodCall class methodsFor!

from: aStream
	^super new from: aStream!

methodName: aString

	^self methodName: aString arguments: nil!

methodName: aString argument: anObject

	^self methodName: aString arguments: (Array with: anObject)!

methodName: aString arguments: aCollection

	"Return an XmlRpcMthodCall object that represents a remote method call with 
	given methodName and arguments. The method call is to be executed by server 
	object associated with the server label"

	^(super new)
		methodName: aString;
		arguments: aCollection! !
!XmlRpcMethodCall class categoriesFor: #from:!public! !
!XmlRpcMethodCall class categoriesFor: #methodName:!public! !
!XmlRpcMethodCall class categoriesFor: #methodName:argument:!public! !
!XmlRpcMethodCall class categoriesFor: #methodName:arguments:!public! !

XmlRpcMethodResult guid: (GUID fromString: '{12C9B22B-F1BF-499E-9B40-08B63D1A040A}')!
XmlRpcMethodResult comment: 'XmlRpcMethodResult represents the result of a remote method call. Converts self from/to proper XML.

Instance Variables:
	result	<Object | anXmlRpcFault>	result of a remote method or an XmlRpcFault if method call failed

'!
!XmlRpcMethodResult categoriesForClass!Kernel-Objects! !
!XmlRpcMethodResult methodsFor!

from: aStream 
	| parser |
	parser := XmlRpcParser on: aStream.
	parser processMethodResultFor: self!

isXmlRpcFault
	^result isXmlRpcFault!

printXmlRpcOn: aStream 
	aStream
		nextPutAll: XmlRpcParser xmlVersion;
		nextPutAll: '<methodResponse>'.
	
	result isXmlRpcFault 
		ifTrue: [ aStream nextPutXmlRpc: result ]
		ifFalse: [
			aStream 
				nextPutAll: '<params><param><value>';
				nextPutXmlRpc: result;
				nextPutAll: '</value></param></params>'. ].
	aStream nextPutAll: '</methodResponse>'!

result
	^result!

result: anObject 
	result := anObject! !
!XmlRpcMethodResult categoriesFor: #from:!public! !
!XmlRpcMethodResult categoriesFor: #isXmlRpcFault!public! !
!XmlRpcMethodResult categoriesFor: #printXmlRpcOn:!public! !
!XmlRpcMethodResult categoriesFor: #result!public! !
!XmlRpcMethodResult categoriesFor: #result:!public! !

!XmlRpcMethodResult class methodsFor!

errorCode: anInteger description: aString

	"Return an XMlRpcMethodResult for an error in processing a method "

	| fault |

	fault := XmlRpcFault errorCode: anInteger description: aString.
	^self result: fault!

from: aStream

	^super new from: aStream!

result: anObject

	^super new result: anObject! !
!XmlRpcMethodResult class categoriesFor: #errorCode:description:!public! !
!XmlRpcMethodResult class categoriesFor: #from:!public! !
!XmlRpcMethodResult class categoriesFor: #result:!public! !

XmlRpcParser guid: (GUID fromString: '{10ED7070-2711-4992-8A3F-4B7F736E5B31}')!
XmlRpcParser comment: 'Simple parser for XML-RPC XML. Since XML-RPC XML is so simple, implemeted parser from scratch to 
aid in porting code. This may be a mistake.

Instance Variables:
	source	<ReadStream>	 source of XML text to be parsed

Class Variables:
	PredefinedEntities	<Dictionary>	Standard XML predefined entities

'!
!XmlRpcParser categoriesForClass!Kernel-Objects! !
!XmlRpcParser methodsFor!

atEnd

	^source atEnd!

endTag: aString

	"tag <cat>, then aString = cat"

	source
		skipToAll: '</' , aString;
		skipThrough: $>!

nextStartTag

	source skipThrough: $<.
	[ '/?!!' includes: source peek ] whileTrue: [ source skipThrough: $< ].
	^source atEnd ifTrue: [ nil ] ifFalse: [ source upTo: $> ]!

peek

	^source peek!

peekNextTag

	| startPosition tag |

	startPosition := source position.
	source skipThrough: $<.
	[ '?!!' includes: source peek ] whileTrue: [ source skipThrough: $< ].
	tag := source upTo: $>.
	source position: startPosition.
	^tag!

processArray

	| array expectedTag |

	array := OrderedCollection new.
	expectedTag := self nextStartTag.
	(expectedTag = 'data' or: [ expectedTag = 'data /' ])
		ifFalse: [ XmlRpcParseError signal: 'Missing <data> tag inside <array> tag' ].
	[ self peekNextTag = 'value' ]
		whileTrue:
			[ self nextStartTag.
			array addLast: self processValue.
			self endTag: 'value' ].
	self endTag: 'array'.
	^array!

processBase64

	^self tagContents decodeBase64!

processBoolean

	| value val |

	value := self tagContents trimSeparators.
	value = '0' ifTrue: [ ^false ].
	value = '1' ifTrue: [ ^true ].
	val := value trimBlanks asLowercase.
	val = 'true' ifTrue: [ ^true ].
	val = 'false' ifTrue: [ ^false ].
	XmlRpcParseError signal: 'Invalid value ' , value , ' for boolean'!

processDateTime

	| value |

	value := self tagContents trimSeparators.
	^TimeStamp fromISO8601: value!

processDouble

	^self tagContents asNumber!

processFault

	| faultDictionary |

	self nextStartTag = 'value'
		ifFalse: [ XmlRpcParseError signal: 'Missing <value> tag inside <fault> tag' ].
	faultDictionary := self processValue.
	^XmlRpcFault
		errorCode: (faultDictionary at: 'faultCode')
		description: (faultDictionary at: 'faultString')!

processInteger

	^self tagContents asNumber!

processMember

	| tag key value |

	2 timesRepeat:
			[ tag := self nextStartTag.
			tag = 'name'
				ifTrue: [ key := self processName ]
				ifFalse:
					[ tag = 'value'
						ifTrue: [ value := self processValue ]
						ifFalse: [ XmlRpcParseError signal: 'Invalid tag ' , tag , ' in <member>' ] ] ].
	self endTag: 'member'.
	^Association key: key value: value!

processMethodCallFor: anRpcXmlMethodCall

	| tag labelAndName arguments |

	self nextStartTag = 'methodCall'
		ifFalse: [ XmlRpcParseError signal: 'Missing <methodCall> tag' ].
	2 timesRepeat:
			[ tag := self nextStartTag.
			tag = 'methodName' ifTrue: [ labelAndName := self processMethodName ].
			tag = 'params' ifTrue: [ arguments := self processParams ] ].
	arguments := arguments isNil ifTrue: [ nil ] ifFalse: [ arguments asArray ].
	labelAndName ifNil: [ XmlRpcParseError signal: 'Missing <methodName> tag' ].
	anRpcXmlMethodCall
		methodName: labelAndName;
		arguments: arguments!

processMethodName

	| contents |

	contents := self tagContents.
	self
		endTag: 'methodName';
		skipWhiteSpace.
	^contents!

processMethodResultFor: anRpcXmlMethodResult

	| tag result |

	self nextStartTag = 'methodResponse'
		ifFalse: [ XmlRpcParseError signal: 'Missing <methodResponse> tag' ].
	tag := self nextStartTag.
	result := tag = 'params'
		ifTrue: [ self processParams first ]
		ifFalse:
			[ tag = 'fault'
				ifTrue: [ self processFault ]
				ifFalse: [ XmlRpcParseError signal: 'Invalid tag ' , tag , ' in method response' ] ].
	anRpcXmlMethodResult result: result!

processName

	^self tagContents decodeXmlRpc!

processObject

	"Quick hack to experiment with sending all objects in XML-RPC"

	| className variables |

	self nextStartTag = 'class'
		ifTrue: [ className := self processName ]
		ifFalse: [ XmlRpcParseError signal: 'Missing <className> in <object>' ].
	self nextStartTag = 'instanceVariables'
		ifTrue:
			[ self nextStartTag.
			variables := self processStruct ]
		ifFalse: [ XmlRpcParseError signal: 'Missing <instanceVariables> in <object>' ].
	^(Smalltalk at: className asSymbol) xmlRpcNew: variables!

processParam

	| tag value |

	tag := self nextStartTag.
	value := tag = 'value'
		ifTrue: [ self processValue ]
		ifFalse: [ XmlRpcParseError signal: 'Invalid tag ' , tag , ' in <param>' ].
	self endTag: 'param'.
	^value!

processParams

	| arguments |

	arguments := OrderedCollection new.
	[ self peekNextTag = 'param' ]
		whileTrue:
			[ self nextStartTag.
			arguments addLast: self processParam ].
	self endTag: 'params'.
	^arguments asArray!

processString

	^self tagContents decodeXmlRpc!

processStruct

	| struct |

	struct := Dictionary new.
	[ self peekNextTag = 'member' ]
		whileTrue:
			[ self nextStartTag.
			struct add: self processMember ].
	self endTag: 'struct'.
	^struct!

processValue

	| tag |

	self peekNextTag = '/value' ifTrue: [ ^self processString ].
	tag := self nextStartTag.
	tag isXmlRpcInt ifTrue: [ ^self processInteger ].
	tag isXmlRpcBoolean ifTrue: [ ^self processBoolean ].
	tag isXmlRpcString ifTrue: [ ^self processString ].
	tag isXmlRpcDouble ifTrue: [ ^self processDouble ].
	tag isXmlRpcArray ifTrue: [ ^self processArray ].
	tag isXmlRpcStruct ifTrue: [ ^self processStruct ].
	tag isXmlRpcDateTime ifTrue: [ ^self processDateTime ].
	tag isXmlRpcBase64 ifTrue: [ ^self processBase64 ].
	tag isXmlRpcObject ifTrue: [ ^self processObject ].
	tag isXmlRpcNil ifTrue: [ ^nil ].
	XmlRpcParseError signal: 'Invalid tag ' , tag , ' in <param>'!

skipComment

	source
		skipToAll: '-->';
		skip: 3!

skipWhiteSpace

	source skipSeparators!

source: aStream

	source := aStream!

tagContents

	| contents |

	contents := source upTo: $<.	"upTo: leaves stream after occurance of $<"
	source skip: -1.
	^contents! !
!XmlRpcParser categoriesFor: #atEnd!public! !
!XmlRpcParser categoriesFor: #endTag:!public! !
!XmlRpcParser categoriesFor: #nextStartTag!public! !
!XmlRpcParser categoriesFor: #peek!public! !
!XmlRpcParser categoriesFor: #peekNextTag!public! !
!XmlRpcParser categoriesFor: #processArray!public! !
!XmlRpcParser categoriesFor: #processBase64!public! !
!XmlRpcParser categoriesFor: #processBoolean!public! !
!XmlRpcParser categoriesFor: #processDateTime!public! !
!XmlRpcParser categoriesFor: #processDouble!public! !
!XmlRpcParser categoriesFor: #processFault!public! !
!XmlRpcParser categoriesFor: #processInteger!public! !
!XmlRpcParser categoriesFor: #processMember!public! !
!XmlRpcParser categoriesFor: #processMethodCallFor:!public! !
!XmlRpcParser categoriesFor: #processMethodName!public! !
!XmlRpcParser categoriesFor: #processMethodResultFor:!public! !
!XmlRpcParser categoriesFor: #processName!public! !
!XmlRpcParser categoriesFor: #processObject!public! !
!XmlRpcParser categoriesFor: #processParam!public! !
!XmlRpcParser categoriesFor: #processParams!public! !
!XmlRpcParser categoriesFor: #processString!public! !
!XmlRpcParser categoriesFor: #processStruct!public! !
!XmlRpcParser categoriesFor: #processValue!public! !
!XmlRpcParser categoriesFor: #skipComment!public! !
!XmlRpcParser categoriesFor: #skipWhiteSpace!public! !
!XmlRpcParser categoriesFor: #source:!public! !
!XmlRpcParser categoriesFor: #tagContents!public! !

!XmlRpcParser class methodsFor!

entityAt: aString

	^PredefinedEntities at: aString!

entityAt: aString ifAbsent: aBlock 

	^PredefinedEntities at: aString ifAbsent: aBlock!

entityAt: aString put: entityString

	"RpcXmlParser entityAt: 'lt' put: '<' "

	^PredefinedEntities at: aString put: entityString!

initialize

	"XmlRpcParser initialize"

	"Standard entites defined in XML 1.0 spec, section 4.6 Predefined Entities"

	PredefinedEntities := Dictionary new.
	self
		entityAt: 'lt' put: '<';
		entityAt: '#60' put: '<';
		entityAt: 'gt' put: '>';
		entityAt: '#62' put: '>';
		entityAt: 'amp' put: '&';
		entityAt: '#38' put: '&';
		entityAt: 'apos' put: '''';
		entityAt: '#39' put: '''';
		entityAt: 'quot' put: '"';
		entityAt: '#34' put: '"'!

on: aStream

	^self new source: aStream!

xmlVersion

	^'<?xml version="1.0"?>'! !
!XmlRpcParser class categoriesFor: #entityAt:!public! !
!XmlRpcParser class categoriesFor: #entityAt:ifAbsent:!public! !
!XmlRpcParser class categoriesFor: #entityAt:put:!public! !
!XmlRpcParser class categoriesFor: #initialize!public! !
!XmlRpcParser class categoriesFor: #on:!public! !
!XmlRpcParser class categoriesFor: #xmlVersion!public! !

XmlRpcProxy guid: (GUID fromString: '{C81C0FFB-A394-4B86-8DBC-3F46380487B2}')!
XmlRpcProxy comment: ''!
!XmlRpcProxy categoriesForClass!Kernel-Objects! !
!XmlRpcProxy methodsFor!

client

	^client!

doesNotUnderstand: aMessage

	"(self extractRpcMethod: aMessage) inspect"

	^self performXmlRpc: aMessage!

extractMethodName: aMessage

	^aMessage selector readStream upTo: $:!

extractRpcMethod: aMessage

	| aStream |

	aStream := WriteStream on: (String new: 64).
	self namespaceAvailable
		ifTrue:
			[ aStream
				nextPutAll: self namespace;
				nextPut: $. ].
	aStream nextPutAll: (self extractMethodName: aMessage).
	^aStream contents!

namespace

	^namespace!

namespaceAvailable

	^self namespace notNil and: [ self namespace size > 0 ]!

performXmlRpc: aMessage

	^self client perform: (self extractRpcMethod: aMessage) withArguments: aMessage arguments!

setClient: anXmlRpcClient namespace: aStringOrNil

	client := anXmlRpcClient.
	namespace := aStringOrNil! !
!XmlRpcProxy categoriesFor: #client!public! !
!XmlRpcProxy categoriesFor: #doesNotUnderstand:!public! !
!XmlRpcProxy categoriesFor: #extractMethodName:!private! !
!XmlRpcProxy categoriesFor: #extractRpcMethod:!private! !
!XmlRpcProxy categoriesFor: #namespace!public! !
!XmlRpcProxy categoriesFor: #namespaceAvailable!private! !
!XmlRpcProxy categoriesFor: #performXmlRpc:!private! !
!XmlRpcProxy categoriesFor: #setClient:namespace:!private! !

!XmlRpcProxy class methodsFor!

example
	| blogger result |
	result := Array new: 3.
	blogger := XmlRpcProxy url: 'http://plant.blogger.com/api/RPC2' namespace: 'blogger'.
	result at: 1
		put: (blogger 
				getUserInfo: '0123456789ABCDEF'
				username: 'user'
				password: 'pass').
	result at: 2
		put: (blogger 
				getUsersBlogs: '0123456789ABCDEF'
				username: 'user'
				password: 'pass').
	result at: 3
		put: (blogger 
				getRecentPosts: '0123456789ABCDEF'
				blogId: '1'
				username: 'user'
				password: 'pass'
				numberOfPosts: 1).
	^result!

on: anXmlRpcClient

	^self on: anXmlRpcClient namespace: nil!

on: anXmlRpcClient namespace: aStringOrNil

	^self new setClient: anXmlRpcClient namespace: aStringOrNil!

server: aServer path: aPath port: aPort namespace: aNamespace

	^self new!

url: aString 
	^self on: MockXmlRpcClient new!

url: aString namespace: aStringOrNil

	^self on: (XmlRpcClient url: aString) namespace: aStringOrNil! !
!XmlRpcProxy class categoriesFor: #example!public! !
!XmlRpcProxy class categoriesFor: #on:!public! !
!XmlRpcProxy class categoriesFor: #on:namespace:!public! !
!XmlRpcProxy class categoriesFor: #server:path:port:namespace:!public! !
!XmlRpcProxy class categoriesFor: #url:!public! !
!XmlRpcProxy class categoriesFor: #url:namespace:!public! !

XmlRpcParseError guid: (GUID fromString: '{11471FFB-1DE6-4F17-8AED-D2AD5E7F24BE}')!
XmlRpcParseError comment: 'XmlRpcParseError is the exception thrown when a error occurs in parsing XMLRPC XML.
'!
!XmlRpcParseError categoriesForClass!Kernel-Exception Handling! !
XmlRpcRemoteException guid: (GUID fromString: '{5D17D8A1-4406-4010-9034-16C57BB6B8F0}')!
XmlRpcRemoteException comment: 'XmlRpcRemoteException thrown by XmlRpcProxy when an error occurs in executing a remote method.

'!
!XmlRpcRemoteException categoriesForClass!Kernel-Exception Handling! !
"Binary Globals"!

"Resources"!

